#!/usr/bin/env python3

"""
picast - a simple wireless display receiver for Raspberry Pi

    Copyright (C) 2019 Hiroshi Miura
    Copyright (C) 2018 Hsun-Wei Cho

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import argparse
import os
from logging import config as LoggingConfig
from logging import getLogger
from typing import Any, Optional, Union

import gi

os.putenv("DISPLAY", ":0")  # noqa: E402 # isort:skip
gi.require_version("Gst", "1.0")  # noqa: E402 # isort:skip
gi.require_version("Gtk", "3.0")  # noqa: E402 # isort:skip
gi.require_version("GstVideo", "1.0")  # noqa: E402 # isort:skip
gi.require_version("GdkX11", "3.0")  # noqa: E402 # isort:skip

from .rtspsink import RtspSink  # noqa: E402 # isort:skip
from .player import GstPlayer, VlcPlayer, NopPlayer  # noqa: E402 # isort:skip
from .settings import Settings  # noqa: E402 # isort:skip
from .wifip2p import WifiP2PServer  # noqa: E402 # isort:skip


def main(arg: Optional[Any] = None):
    parser = argparse.ArgumentParser(
        prog="picast", description="picast", formatter_class=argparse.RawTextHelpFormatter, add_help=True
    )
    parser.add_argument("--debug", action="store_true", help="Verbose debug output")
    parser.add_argument("-c", "--config", help="Specify configuration file.")

    args = parser.parse_args(arg)
    if args.config:
        config_file = args.config
        if os.path.exists(config_file) and os.path.isfile(config_file):
            pass
        else:
            config_file = None
    else:
        config_file = None

    # ------------------- start of configurations
    # it should call first: load configurations from ini files
    config = Settings(config=config_file)
    package_dir = os.path.join(os.path.dirname(__file__))
    if config.logging_config:
        logging_ini = config.logging_config
        if not os.path.exists(logging_ini):
            if os.path.exists(os.path.join(package_dir, logging_ini)):
                logging_ini = os.path.join(package_dir, logging_ini)
    else:
        logging_ini = os.path.join(package_dir, "logging.ini")
    LoggingConfig.fileConfig(logging_ini)
    logger = getLogger(config.logger)
    if args.debug:
        logger.setLevel("DEBUG")

    if config.player == "gst":
        player: Optional[Union[GstPlayer, VlcPlayer, NopPlayer]] = GstPlayer()
    elif config.player == "vlc":
        player = VlcPlayer()
    elif config.player == "nop":
        player = NopPlayer()
    else:
        player = None
        logger.fatal("FATAL: Unknown player name option!: {}".format(config.player))
        exit(1)
    # ------------------- end of configurations

    wifip2p = WifiP2PServer()
    rtspsink = RtspSink(player)

    wifip2p.start()
    rtspsink.start()

    rtspsink.join()
    wifip2p.join()


if __name__ == "__main__":
    main()



# Configuration for pytest to automatically collect types.
# Thanks to Guilherme Salgado.
import socket

import pytest
from pyannotate_runtime import collect_types


def pytest_collection_finish(session):
    """Handle the pytest collection finish hook: configure pyannotate.
    Explicitly delay importing `collect_types` until all tests have
    been collected.  This gives gevent a chance to monkey patch the
    world before importing pyannotate.
    """
    collect_types.init_types_collection()


@pytest.fixture(autouse=True)
def collect_types_fixture():
    collect_types.start()
    yield
    collect_types.stop()


def pytest_sessionfinish(session, exitstatus):
    collect_types.dump_stats("type_info.json")


@pytest.fixture(scope="function")
def unused_port():
    s = socket.socket(socket.AF_INET, type=socket.SOCK_STREAM)
    s.bind(('localhost', 0))
    _, port = s.getsockname()
    s.close()
    return port



#!/usr/bin/env python3

"""
picast - a simple wireless display receiver for Raspberry Pi

    Copyright (C) 2019,2020 Hiroshi Miura
    Copyright (C) 2018 Hsun-Wei Cho

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import socket
from logging import getLogger

import zeroconf

from .settings import Settings


class ServiceDiscovery:
    """Register and query mDNS/SD entries"""

    def __init__(self):
        self.config = Settings()
        self.logger = getLogger(self.config.logger)
        self.zc = zeroconf.Zeroconf()

    def register(self):
        service_info = zeroconf.ServiceInfo(
            "_display._tcp.local.",
            "PiCast Remote Display._display._tcp.local.",
            addresses=[socket.inet_aton(self.config.myaddress)],
            port=self.config.rtsp_port,
        )
        self.zc.register_service(service_info, ttl=60, allow_name_change=False)
        self.logger.info("Register mDNS/SD entry.")

    def lookup(self):
        service_info = self.zc.get_service_info("_displaysrc._tcp.local.", None)
        return service_info.addresses, service_info.port




from picast.video import RasberryPiVideo
from picast.wifip2p import WifiP2PServer
from picast.wpacli import WpaCli

import pytest


def _get_display_resolutions_mock(obj):
    obj.cea = 0x0101C3
    obj.vesa = 0x0208006
    obj.hh = 0x00


@pytest.mark.unit
def test_get_video_parameter(monkeypatch):
    def mockreturn(self):
        return _get_display_resolutions_mock(self)

    monkeypatch.setattr(RasberryPiVideo, "_get_display_resolutions", mockreturn)

    expected = "06 00 01 10 000101C3 00208006 00000000 00 0000 0000 00 none none"
    wvp = RasberryPiVideo()
    assert wvp.get_wfd_video_formats() == expected


@pytest.mark.unit
def test_wpacli_start_p2p_find(monkeypatch):

    def mockreturn(self, *args):
        assert args == ('p2p_find', 'type=progressive',)
        return "OK"

    monkeypatch.setattr(WpaCli, "cmd", mockreturn)
    wpacli = WpaCli()
    wpacli.start_p2p_find()


@pytest.mark.unit
def test_wpacli_stop_p2p_find(monkeypatch):

    def mockreturn(self, *arg):
        assert arg == ('p2p_stop_find',)
        return "OK"

    monkeypatch.setattr(WpaCli, "cmd", mockreturn)
    wpacli = WpaCli()
    wpacli.stop_p2p_find()


@pytest.mark.unit
def test_wpacli_set_device_name(monkeypatch):

    def mockreturn(self, *arg):
        assert arg == ("set", "device_name", "foo",)
        return "OK"

    monkeypatch.setattr(WpaCli, "cmd", mockreturn)
    wpacli = WpaCli()
    wpacli.set_device_name("foo")


@pytest.mark.unit
def test_wpacli_set_device_type(monkeypatch):

    def mockreturn(self, *arg):
        assert arg == ("set", "device_type", "foo",)
        return "OK"

    monkeypatch.setattr(WpaCli, "cmd", mockreturn)
    wpacli = WpaCli()
    wpacli.set_device_type("foo")


@pytest.mark.unit
def test_wpacli_set_p2p_go_ht40(monkeypatch):

    def mockreturn(self, *arg):
        assert arg == ("set", "p2p_go_ht40", "1",)
        return "OK"

    monkeypatch.setattr(WpaCli, "cmd", mockreturn)
    wpacli = WpaCli()
    wpacli.set_p2p_go_ht40()


@pytest.mark.unit
def test_wpacli_wfd_subelem_set(monkeypatch):

    def mockreturn(self, *arg):
        assert arg == ("wfd_subelem_set", "0", '00000000')
        return "OK"

    monkeypatch.setattr(WpaCli, "cmd", mockreturn)
    wpacli = WpaCli()
    wpacli.wfd_subelem_set(0, "00000000")


@pytest.mark.unit
def test_wpacli_p2p_group_add(monkeypatch):

    def mockreturn(self, *arg):
        assert arg == ("p2p_group_add", 'group')
        return "OK"

    monkeypatch.setattr(WpaCli, "cmd", mockreturn)
    wpacli = WpaCli()
    wpacli.p2p_group_add("group")


@pytest.mark.unit
def test_wpacli_wps_pin(monkeypatch):

    def mockreturn(self, *arg):
        assert arg == ('-i', 'w1p0', 'wps_pin', 'any', '12345678', '300')
        return "OK"

    monkeypatch.setattr(WpaCli, "cmd", mockreturn)
    wpacli = WpaCli()
    wpacli.set_wps_pin("w1p0", '12345678', 300)


@pytest.mark.unit
def test_wpa_p2p_interface(monkeypatch):
    def mockreturn(self, *arg):
        assert arg == ('interface',)
        return ["Selected interface 'p2p-wlp4s0'", "Available interfaces:", "p2p-wlp4s0", "wlp4s0"]
    monkeypatch.setattr(WpaCli, "cmd", mockreturn)
    wpacli = WpaCli()
    result = wpacli.get_p2p_interface()
    assert result == "p2p-wlp4s0"


@pytest.mark.unit
def test_wpa_check_p2p_interface(monkeypatch):
    def mockreturn(self, *arg):
        assert arg == ('interface',)
        return ["Selected interface 'p2p-wlp4s0'", "Available interfaces:", "p2p-wlp4s0", "wlp4s0"]
    monkeypatch.setattr(WpaCli, "cmd", mockreturn)
    wpacli = WpaCli()
    assert wpacli.check_p2p_interface()


@pytest.mark.unit
def test_wpa_get_interface(monkeypatch):
    def mockreturn(self, *arg):
        assert arg == ('interface',)
        return ["Selected interface 'p2p-wlp4s0'", "Available interfaces:", "p2p-wlp4s0", "wlp4s0"]
    monkeypatch.setattr(WpaCli, "cmd", mockreturn)
    wpacli = WpaCli()
    selected, interfaces = wpacli.get_interfaces()
    assert selected == 'p2p-wlp4s0'
    assert interfaces == ['p2p-wlp4s0', 'wlp4s0']


@pytest.mark.unit
def test_devinfo(monkeypatch):
    def mockreturn(self, *arg):
        return
    monkeypatch.setattr(WifiP2PServer, "set_p2p_interface", mockreturn)
    p2p = WifiP2PServer()
    assert  p2p.wfd_devinfo() == '00060151022a012c'


@pytest.mark.unit
def test_devinfo2(monkeypatch):
    def mockreturn(self, *arg):
        return
    monkeypatch.setattr(WifiP2PServer, "set_p2p_interface", mockreturn)
    p2p = WifiP2PServer()
    assert p2p.wfd_devinfo2() == '00020001'


@pytest.mark.unit
def test_ext_cap(monkeypatch):
    def mockreturn(self, *arg):
        return
    monkeypatch.setattr(WifiP2PServer, "set_p2p_interface", mockreturn)
    p2p = WifiP2PServer()
    assert p2p.wfd_ext_cap(uibc=False, i2c=False) == '00020000'
    assert p2p.wfd_ext_cap(uibc=True, i2c=False) == '00020001'
    assert p2p.wfd_ext_cap(uibc=False, i2c=True) == '00020002'


def _tvservice_mock(cmd):
    """Return actual modes retrieved from Raspberry Pi Zero WH with FHD monitor."""
    if cmd == "tvservice -m CEA -j":
        return '[{ "code":1, "width":640, "height":480, "rate":60, "aspect_ratio":"4:3", "scan":"p", "3d_modes":[] },' \
               ' { "code":2, "width":720, "height":480, "rate":60, "aspect_ratio":"4:3", "scan":"p", "3d_modes":[] },' \
               ' { "code":3, "width":720, "height":480, "rate":60, "aspect_ratio":"16:9", "scan":"p", "3d_modes":[] },' \
               ' { "code":4, "width":1280, "height":720, "rate":60, "aspect_ratio":"16:9", "scan":"p", "3d_modes":[] },' \
               ' { "code":16, "width":1920, "height":1080, "rate":60, "aspect_ratio":"16:9", "scan":"p", "3d_modes":[] }, ' \
               '{ "code":32, "width":1920, "height":1080, "rate":24, "aspect_ratio":"16:9", "scan":"p", "3d_modes":[] }, ' \
               '{ "code":34, "width":1920, "height":1080, "rate":30, "aspect_ratio":"16:9", "scan":"p", "3d_modes":[] } ]'
    elif cmd == "tvservice -m DMT -j":
        return '[{ "code":4, "width":640, "height":480, "rate":60, "aspect_ratio":"4:3", "scan":"p", "3d_modes":[] },' \
               ' { "code":9, "width":800, "height":600, "rate":60, "aspect_ratio":"4:3", "scan":"p", "3d_modes":[] },' \
               ' { "code":16, "width":1024, "height":768, "rate":60, "aspect_ratio":"4:3", "scan":"p", "3d_modes":[] },' \
               ' { "code":35, "width":1280, "height":1024, "rate":60, "aspect_ratio":"5:4", "scan":"p", "3d_modes":[] },' \
               ' { "code":83, "width":1600, "height":900, "rate":60, "aspect_ratio":"16:9", "scan":"p", "3d_modes":[] },' \
               ' { "code":85, "width":1280, "height":720, "rate":60, "aspect_ratio":"16:9", "scan":"p", "3d_modes":[] }]'
    else:
        return None

@pytest.mark.unit
def test_tvservice_cea(monkeypatch):
    def mockreturn(self, cmd):
        return _tvservice_mock(cmd)
    monkeypatch.setattr(RasberryPiVideo, "_call_tvservice", mockreturn)
    v = RasberryPiVideo()
    param = v._retrieve_tvservice(RasberryPiVideo.TvModes.CEA)
    assert param[0]["code"] == 1
    assert param[1]["width"] == 720
    assert param[2]["height"] == 480
    assert param[3]["rate"] == 60


@pytest.mark.unit
def test_tvservice_dmt(monkeypatch):
    def mockreturn(self, cmd):
        return _tvservice_mock(cmd)
    monkeypatch.setattr(RasberryPiVideo, "_call_tvservice", mockreturn)
    v = RasberryPiVideo()
    param = v._retrieve_tvservice(RasberryPiVideo.TvModes.DMT)
    assert param[0]["code"] == 4
    assert param[1]["width"] == 800


@pytest.mark.unit
def test_load_resolutons_json(monkeypatch):
    def mockreturn(self):
        return _get_display_resolutions_mock(self)
    monkeypatch.setattr(RasberryPiVideo, "_get_display_resolutions", mockreturn)
    v = RasberryPiVideo()
    assert v.resolutions['cea'] is not None
    assert v.resolutions['vesa'] is not None
    assert len(v.resolutions['cea']) == 12
    assert len(v.resolutions['vesa']) == 8


@pytest.mark.unit
def test_get_display_resolution(monkeypatch):
    def mockreturn(self, cmd):
        return _tvservice_mock(cmd)
    monkeypatch.setattr(RasberryPiVideo, "_call_tvservice", mockreturn)
    v = RasberryPiVideo()
    assert v.cea == 0x0101C3
    assert v.vesa == 0x0208006
    assert v.hh == 0x00




import os
from logging import config as LoggingConfig

from picast.settings import Settings

import pytest


@pytest.mark.unit
def test_logging_config():
    LoggingConfig.fileConfig(os.path.join(os.path.dirname(os.path.dirname(__file__)), 'src', 'picast', 'logging.ini'))


@pytest.mark.unit
def test_config_logger():
    assert Settings().logger == 'picast'


@pytest.mark.unit
def test_config_myaddress():
    assert Settings().myaddress == '192.168.173.1'


@pytest.mark.unit
def test_config_peeraddress():
    assert Settings().peeraddress == '192.168.173.80'


@pytest.mark.unit
def test_config_netmask():
    assert Settings().netmask == '255.255.255.0'


@pytest.mark.unit
def test_config_rtp_port():
    assert Settings().rtp_port == 1028


@pytest.mark.unit
def test_config_rtsp_port():
    assert Settings().rtsp_port == 7236


@pytest.mark.unit
def test_config_device_name():
    assert Settings().device_name == 'picast'


@pytest.mark.unit
def test_config_device_type():
    assert Settings().device_type == '7-0050F204-4'


@pytest.mark.unit
def test_config_group_name():
    assert Settings().group_name == 'persistent'


@pytest.mark.unit
def test_config_pin():
    assert Settings().pin == '12345678'



#!/usr/bin/env python3

"""
picast - a simple wireless display receiver for Raspberry Pi

    Copyright (C) 2019 Hiroshi Miura
    Copyright (C) 2018 Hsun-Wei Cho

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import enum
import json
import os
import re
import subprocess
from logging import getLogger
from typing import Dict, List

from .settings import Settings


class GenericVideo:
    def __init__(self):
        self.native = 0x06  # type: int
        self.preferred = 0  # type: int
        self.profile = 0x01  # type: int
        self.level = 0x10  # type: int
        self.cea = 0x00FFFFFF  # type: int
        self.vesa = 0x00FFFFFF  # type: int
        self.hh = 0x00FF  # type: int

    def get_wfd_video_formats(self) -> str:
        return "{0:02X} {1:02X} {2:02X} {3:02X} {4:08X} {5:08X} {6:08X} 00 0000 0000 00 none none".format(
            self.native, self.preferred, self.profile, self.level, self.cea, self.vesa, self.hh
        )


class RasberryPiVideo:
    """Video parameters from Raspberry Pi specific command."""

    def __init__(self):
        self.config = Settings()
        self.native = 0x06
        self.preferred = 0
        self.profile = 0x01
        self.level = 0x10
        with open(os.path.join(os.path.dirname(__file__), "resolutions.json"), "r") as j:
            self.resolutions = json.load(j)[0]
        self._get_display_resolutions()

    def get_wfd_video_formats(self) -> str:
        return "{0:02X} {1:02X} {2:02X} {3:02X} {4:08X} {5:08X} {6:08X} 00 0000 0000 00 none none".format(
            self.native, self.preferred, self.profile, self.level, self.cea, self.vesa, self.hh
        )

    class TvModes(enum.Enum):
        CEA = "-m CEA -j"
        DMT = "-m DMT -j"
        Current = "-s"

    def _call_tvservice(self, cmd):
        logger = getLogger(self.config.logger)
        data = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE).communicate()[0]
        logger.debug("tvservice: {}".format(data))
        return data

    def _retrieve_tvservice(self, mode: TvModes) -> List[Dict[str, str]]:
        status = []  # type: List[Dict[str, str]]
        if mode is self.TvModes.Current:
            data = self._call_tvservice("tvservice -s")
            r = re.compile(r"([0-9]+)x([0-9]+),\s+@\s+([1-9][0-9])\.[0-9][0-9]HZ")
            m = r.match(data)
            if m is not None:
                status = [{"width": m.group(1), "height": m.group(2), "rate": m.group(3)}]
        elif mode is self.TvModes.CEA:
            data = self._call_tvservice("tvservice -m CEA -j")
            status = json.loads(data)
        else:
            data = self._call_tvservice("tvservice -m DMT -j")
            status = json.loads(data)
        return status

    def _get_display_resolutions(self) -> None:
        cea = 0x01
        vesa = 0x00
        hh = 0x00
        cea_resolutions = self._retrieve_tvservice(mode=self.TvModes.CEA)  # type: List[Dict[str, str]]
        for r in cea_resolutions:
            res_list = self.resolutions["cea"]
            for res in res_list:
                if res["mode"] == r["code"]:
                    cea |= 1 << res["id"]
                    break
        dmt_resolutions = self._retrieve_tvservice(mode=self.TvModes.DMT)  # type: List[Dict[str, str]]
        for r in dmt_resolutions:
            res_list = self.resolutions["vesa"]
            for res in res_list:
                if res["mode"] == r["code"]:
                    vesa |= 1 << res["id"]
                    break
        self.cea = cea
        self.vesa = vesa
        self.hh = hh



#!/usr/bin/env python3

"""
picast - a simple wireless display receiver for Raspberry Pi

    Copyright (C) 2019 Hiroshi Miura
    Copyright (C) 2018 Hsun-Wei Cho

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import os
import threading
from logging import getLogger
from time import sleep

from .dhcpd import Dhcpd
from .exceptions import WpaException
from .settings import Settings
from .wpacli import WpaCli


class WifiP2PServer(threading.Thread):
    def __init__(self, R2=False, logger="picast"):
        super(WifiP2PServer, self).__init__(name="wifi-p2p-0", daemon=False)
        self.config = Settings()
        self.logger = getLogger(logger)
        self.set_p2p_interface(R2)

    def run(self):
        self.start_dhcpd()
        self.start_wps()
        sleep(3)

    def start_wps(self):
        wpacli = WpaCli()
        if self.config.wps_mode == "pbc":
            wpacli.start_wps_pbc(self.wlandev)
        else:
            wpacli.set_wps_pin(self.wlandev, self.config.pin, self.config.timeout)

    def start_dhcpd(self):
        dhcpd = Dhcpd(self.wlandev)
        dhcpd.start()

    def wfd_devinfo(self):
        type = 0b01  # PRIMARY_SINK
        session = 0b01 << 4
        wsd = 0b01 << 6
        pc = 0b00  # = P2P
        cp_support = 0b01 << 8
        ts = 0b00
        devinfo = type | session | wsd | pc | cp_support | ts
        control = 554  # Settings.rtsp_port
        max_tp = 300  # Mbps
        return "0006{0:04x}{1:04x}{2:04x}".format(devinfo, control, max_tp)

    def wfd_bssid(self, bssid: int) -> str:
        return "0006{0:012X}".format(bssid)

    def wfd_sink_info(self, status: int, mac: int) -> str:
        return "0007{0:02X}{1:012X}".format(status, mac)

    def wfd_ext_cap(self, uibc=False, i2c=False) -> str:
        val_uibc = 0b1 if uibc else 0b0
        val_i2c = (0b1 if i2c else 0b0) << 1
        val = val_uibc | val_i2c
        return "0002{0:04X}".format(val)

    def wfd_devinfo2(self) -> str:
        r2_sink = 0b01
        return "0002{0:04X}".format(r2_sink)

    def create_p2p_interface(self, R2):
        wpacli = WpaCli()
        wpacli.start_p2p_find()
        wpacli.set_device_name(self.config.device_name)
        wpacli.set_device_type(self.config.device_type)
        wpacli.set_p2p_go_ht40()
        wpacli.wfd_subelem_set(0, self.wfd_devinfo())
        wpacli.wfd_subelem_set(1, self.wfd_bssid(0))
        wpacli.wfd_subelem_set(6, self.wfd_sink_info(0, 0))
        wpacli.wfd_subelem_set(7, self.wfd_ext_cap(uibc=False, i2c=False))
        if R2:
            wpacli.wfd_subelem_set(11, self.wfd_devinfo2())
        wpacli.p2p_group_add(self.config.group_name)

    def set_p2p_interface(self, R2):
        wpacli = WpaCli()
        if wpacli.check_p2p_interface():
            self.logger.info("Already set a p2p interface.")
            p2p_interface = wpacli.get_p2p_interface()
        else:
            self.create_p2p_interface(R2)
            sleep(3)
            p2p_interface = wpacli.get_p2p_interface()
            if p2p_interface is None:
                interfaces = wpacli.get_interfaces()
                message = "Can not create P2P Wifi interface.\n\nCurrent interfaces are:\n"
                for it in interfaces:
                    message += "{:s} ".format(it)
                message += "\n"
                raise WpaException(message)
            self.logger.info("Start p2p interface: {} address {}".format(p2p_interface, self.config.myaddress))
            os.system("sudo ifconfig {} {}".format(p2p_interface, self.config.myaddress))
        self.wlandev = p2p_interface




import pytest

from picast.dhcpd import Dhcpd


@pytest.mark.unit
def test_dhcpd_conf():
    dhcpd = Dhcpd('p2p-wlan0-0')
    with open(dhcpd.conf_path, 'r') as c:
        conf = c.readlines()
    assert conf == ["start  192.168.173.80\n", "end 192.168.173.80\n",
                    "interface p2p-wlan0-0\n", "notify_file dumpleases\n",
                    "option subnet 255.255.255.0\n", "option lease 300\n"]



#!/usr/bin/env python3

"""
picast - a simple wireless display receiver for Raspberry Pi

    Copyright (C) 2019 Hiroshi Miura
    Copyright (C) 2018 Hsun-Wei Cho

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import os
import subprocess
from logging import getLogger

import gi

os.putenv("DISPLAY", ":0")  # noqa: E402 # isort:skip
gi.require_version("Gst", "1.0")  # noqa: E402 # isort:skip
gi.require_version("Gtk", "3.0")  # noqa: E402 # isort:skip
gi.require_version("GstVideo", "1.0")  # noqa: E402 # isort:skip
gi.require_version("GdkX11", "3.0")  # noqa: E402 # isort:skip
from gi.repository import Gst  # noqa: E402 # isort:skip

from .settings import Settings  # noqa: E402 # isort:skip


class NopPlayer:
    def __init__(self, logger="picast"):
        self.logger = getLogger(logger)
        self.proc = None

    def start(self):
        self.logger.debug("Start nop client.")
        self.proc = subprocess.Popen(["echo", "rtp://0.0.0.0:1028/wfd1.0/streamid=0"])

    def stop(self):
        if self.proc is not None:
            self.logger.debug("Stop nop client.")
            self.proc.terminate()


class VlcPlayer:
    def __init__(self, logger="picast"):
        self.config = Settings()
        self.logger = getLogger(logger)
        self.vlc = None

    def start(self):
        self.logger.debug("Start vlc client.")
        self.vlc = subprocess.Popen(
            [
                "cvlc",
                "--fullscreen",
                "--file-logging",
                "--logfile",
                self.config.player_log_file,
                "rtp://0.0.0.0:1028/wfd1.0/streamid=0",
            ]
        )

    def stop(self):
        if self.vlc is not None:
            self.logger.debug("Stop vlc client.")
            self.vlc.terminate()


class GstPlayer:
    def __init__(self, logger="picast"):
        self.config = Settings()
        self.logger = getLogger(logger)
        Gst.init(None)

    def start(self):
        self.pipeline = Gst.Pipeline()

        src = Gst.ElementFactory.make("udpsrc")
        src.set_property("port", self.config.rtp_port)
        src.set_property("caps", "application/x-rtp, media=video")

        h264 = Gst.ElementFactory.make("rtph264depay")
        omxdecode = Gst.ElementFactory.make(self.config.gst_decoder)
        vconv = Gst.ElementFactory.make("videoconvert")
        sink = Gst.ElementFactory.make("autovideosink")

        for ele in [src, h264, omxdecode, vconv, sink]:
            self.pipeline.add(ele)

        src.link(h264)
        h264.link(omxdecode)
        omxdecode.link(vconv)
        vconv.link(sink)

        self.bus = self.pipeline.get_bus()
        self.bus.add_signal_watch()
        self.bus.connect("message", self.on_message)
        self.logger.debug("Start gst player...")
        self.pipeline.set_state(Gst.State.PLAYING)

    def stop(self):
        self.logger.debug("Stop gst player...")
        self.pipeline.set_state(Gst.State.NULL)

    def on_message(self, bus, message):
        mtype = message.type
        if mtype == Gst.MessageType.EOS:
            self.pipeline.seek_simple(Gst.Format.TIME, Gst.SeekFlags.FLUSH | Gst.SeekFlags.KEY_UNIT, 0)
        elif mtype == Gst.MessageType.ERROR:
            if message.get_structure().get_name() == "prepare-window-handle":
                if hasattr(self, "xid"):
                    message.src.set_window_handle(self.xid)
        elif mtype == Gst.MessageType.WARNING:
            self.logger.debug("on_error():{}".format(message.parse_error()))

        return True



#!/usr/bin/env python3

"""
picast - a simple wireless display receiver for Raspberry Pi

    Copyright (C) 2019 Hiroshi Miura
    Copyright (C) 2018 Hsun-Wei Cho

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import configparser
import enum
import os
import threading


class PlatformType(enum.Enum):
    RaspberryPi = 1
    BeagleBoardBlack = 2
    PandaBoard = 3
    Generic = 999


class Settings(object):
    # this class is Borg/Singleton
    _shared_state = {"_config": None, "_lock": threading.Lock()}

    def __init__(self, config=None):
        self.__dict__ = self._shared_state
        if self._config is None:
            with self._lock:
                if self._config is None:
                    if config is None:
                        inidir = os.path.dirname(__file__)
                        self.inifile = os.path.join(inidir, "settings.ini")
                    else:
                        inidir = os.path.dirname(config)
                        self.inifile = config
                    self._config = self.configParse(self.inifile)
                    self._platform = self._detect_platform()

    def configParse(self, file_path):
        if not os.path.exists(file_path):
            raise IOError(file_path)
        config = configparser.ConfigParser()
        config.read(file_path)
        return config

    def _detect_platform(self):
        self.platform = PlatformType.RaspberryPi  # FIXME

    def get_wfd_parameters(self):
        return self._config["wfd_parameter"]

    @property
    def logging_config(self):
        return self._config.get("logging", "config")

    @property
    def logger(self):
        return self._config.get("logging", "logger")

    @property
    def player(self):
        return self._config.get("player", "name")

    @property
    def player_log_file(self):
        return self._config.get("player", "log_file")

    @property
    def rtp_port(self):
        return self._config.getint("network", "rtp_port")

    @property
    def myaddress(self):
        return self._config.get("network", "myaddress")

    @property
    def peeraddress(self):
        return self._config.get("network", "peeraddress")

    @property
    def netmask(self):
        return self._config.get("network", "netmask")

    @property
    def wps_mode(self):
        return self._config.get("p2p", "wps_mode")

    @property
    def pin(self):
        return self._config.get("p2p", "pin")

    @property
    def timeout(self):
        return self._config.getint("p2p", "timeout")

    @property
    def group_name(self):
        return self._config.get("p2p", "group_name")

    @property
    def device_type(self):
        return self._config.get("p2p", "device_type")

    @property
    def device_name(self):
        return self._config.get("p2p", "device_name")

    @property
    def rtsp_port(self):
        return self._config.getint("network", "rtsp_port")

    @property
    def gst_decoder(self):
        return self._config.get("gst", "decoder")

    @property
    def max_timeout(self):
        return self._config.get("network", "max_timeout")



#!/usr/bin/env python3

"""
picast - a simple wireless display receiver for Raspberry Pi

    Copyright (C) 2019 Hiroshi Miura
    Copyright (C) 2018 Hsun-Wei Cho

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re
import subprocess
from logging import getLogger
from typing import List, Optional, Tuple

from .exceptions import WpaException


class WpaCli:
    """
    Wraps the wpa_cli command line interface.
    """

    def __init__(self, logger="picast"):
        self.logger = getLogger(logger)
        pass

    def cmd(self, *argv):
        p = subprocess.Popen(["sudo", "wpa_cli"] + list(argv), stdout=subprocess.PIPE)
        stdout = p.communicate()[0]
        return stdout.decode("UTF-8").splitlines()

    def start_p2p_find(self):
        self.logger.debug("wpa_cli p2p_find type=progressive")
        status = self.cmd("p2p_find", "type=progressive")
        if "OK" not in status:
            raise WpaException("Fail to start p2p find.")

    def stop_p2p_find(self):
        self.logger.debug("wpa_cli p2p_stop_find")
        status = self.cmd("p2p_stop_find")
        if "OK" not in status:
            raise WpaException("Fail to stop p2p find.")

    def set_device_name(self, name: str):
        self.logger.debug("wpa_cli set device_name {}".format(name))
        status = self.cmd("set", "device_name", name)
        if "OK" not in status:
            raise WpaException("Fail to set device name {}".format(name))

    def set_device_type(self, type):
        self.logger.debug("wpa_cli set device_type {}".format(type))
        status = self.cmd("set", "device_type", type)
        if "OK" not in status:
            raise WpaException("Fail to set device type {}".format(type))

    def set_p2p_go_ht40(self):
        self.logger.debug("wpa_cli set p2p_go_ht40 1")
        status = self.cmd("set", "p2p_go_ht40", "1")
        if "OK" not in status:
            raise WpaException("Fail to set p2p_go_ht40")

    def wfd_subelem_set(self, key: int, val: str):
        self.logger.debug("wpa_cli wfd_subelem_set {0:d} {1:s}".format(key, val))
        status = self.cmd("wfd_subelem_set", "{0:d}".format(key), val)
        if "OK" not in status:
            raise WpaException("Fail to wfd_subelem_set.")

    def p2p_group_add(self, name: str):
        self.logger.debug("wpa_cli p2p_group_add {}".format(name))
        self.cmd("p2p_group_add", name)

    def set_wps_pin(self, interface: str, pin: str, timeout: int):
        self.logger.debug("wpa_cli -i {0:s} wps_pin any {1:s} {2:d}".format(interface, pin, timeout))
        status = self.cmd("-i", interface, "wps_pin", "any", "{0:s}".format(pin), "{0:d}".format(timeout))
        return status

    def start_wps_pbc(self, interface: str):
        self.logger.debug("wpa_cli -i {0:s} wps_pbc".format(interface))
        status = self.cmd("-i", interface, "wps_pbc")
        return status

    def p2p_connect(self, interface: str, pin: str, peer: str):
        self.logger.debug("wpa_cli -i {0:s} p2p_connect {1:s} {2:s}".format(interface, peer, pin))
        status = self.cmd("-i", interface, "p2p_connect", "{0:s}".format(peer), "{0:s}".format(pin))
        return status

    def get_interfaces(self) -> Tuple[Optional[str], List[str]]:
        selected = None
        interfaces = []
        status = self.cmd("interface")
        for ln in status:
            if ln.startswith("Selected interface"):
                m = re.match(r"Selected interface\s\'(.+)\'$", ln)
                if m is not None:
                    selected = m.group(1)
            elif ln.startswith("Available interfaces:"):
                pass
            else:
                interfaces.append(str(ln))
        return selected, interfaces

    def get_p2p_interface(self) -> Optional[str]:
        sel, interfaces = self.get_interfaces()
        for it in interfaces:
            if it.startswith("p2p-") and not it.startswith("p2p-dev"):
                return it
        return None

    def check_p2p_interface(self) -> bool:
        if self.get_p2p_interface() is not None:
            return True
        return False



#!/usr/bin/env python3

"""
picast - a simple wireless display receiver for Raspberry Pi

    Copyright (C) 2019 Hiroshi Miura
    Copyright (C) 2018 Hsun-Wei Cho

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import errno
import re
import socket
import threading
from logging import getLogger
from time import sleep
from typing import Dict, List, Optional, Tuple

from .discovery import ServiceDiscovery
from .settings import Settings
from .video import RasberryPiVideo


class RTSPTransport:
    def __init__(self, host, port):
        self.buffer = None
        self._max_attempt = 1000
        self.open_connection(host, port)

    def open_connection(self, host, port):
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)

        for _ in range(self._max_attempt):
            try:
                sock.connect((host, port))
            except Exception:
                sleep(0.5)
            else:
                break
        sock.settimeout(1)
        self.sock = sock

    def settimeout(self, value):
        return self.sock.settimeout(value)

    def close(self):
        self.sock.close()

    def read(self, size) -> bytes:
        if self.buffer:
            if len(self.buffer) > size:
                res = self.buffer[:size]
                self.buffer = self.buffer[size:]
            else:
                res = self.buffer
                self.buffer = None
                if len(res) < size:
                    res += self.sock.recv(size - len(res))
        else:
            res = self.sock.recv(size)
        return res

    def readline(self) -> bytes:
        if self.buffer is None:
            self.buffer = self.sock.recv(1024)
        while True:
            if b"\r\n" in self.buffer:
                (line, self.buffer) = self.buffer.split(b"\r\n", 1)
                return line
            else:
                more = self.sock.recv(1024)
                if not more:
                    break
                else:
                    self.buffer += more
        if self.buffer is not None:
            res = self.buffer
            self.buffer = None
            return res
        else:
            return b""

    def write(self, b: bytes):
        return self.sock.sendall(b)


class RtspSink(threading.Thread):
    def __init__(self, player, logger="picast"):
        super(RtspSink, self).__init__(name="rtsp-server-0", daemon=True)
        self.config = Settings()
        self.logger = getLogger(logger)
        self.player = player
        self.watchdog = 0
        self.csnum = 0
        self.daemon = True
        self.video = RasberryPiVideo()
        self.wfd_parameters = self.config.get_wfd_parameters()
        self.wfd_video_formats = self.video.get_wfd_video_formats()

    def get_rtsp_headers(self) -> Dict[str, Optional[str]]:
        headers = self.read_headers()
        results = {}
        firstline = headers[0]
        regex = re.compile(r"RTSP/1.0 ([0-9]+) (\w+([ ]\w)*)")
        if firstline.startswith("RTSP/1.0"):
            m = regex.match(firstline)
            if m:
                status, reason = m.group(1, 2)
                cmd = None
                url = None
                resp = "{} {}".format(status, reason)  # type: Optional[str]
            else:
                raise ValueError
        else:
            cmd, url, version = firstline.split(" ")
            if version != "RTSP/1.0":
                raise ValueError
            resp = None
        results["cmd"] = cmd
        results["url"] = url
        results["resp"] = resp
        for h in headers[1:]:
            pos = h.find(":")
            if not pos:
                break
            key = h[:pos]
            val = h[pos + 2 :]
            results[key] = val
        return results

    def read_headers(self) -> List[str]:
        headers = []

        line = self.sock.readline()
        while not line:
            line = self.sock.readline()

        while line:
            headers.append(line.decode("UTF-8"))
            line = self.sock.readline()
        self.logger.debug("<< {}".format(headers))
        return headers

    def read_body(self, headers) -> bytes:
        length = headers.get("Content-Length", None)
        if length is None:
            return b""
        length = int(length)
        return self.sock.read(length)

    @staticmethod
    def _rtsp_response_header(
        cmd: Optional[str] = None,
        url: Optional[str] = None,
        res: Optional[str] = None,
        seq: Optional[str] = None,
        others: Optional[List[Tuple[str, str]]] = None,
    ) -> str:
        if cmd is not None:
            msg = "{0:s} {1:s} RTSP/1.0".format(cmd, url)
        else:
            msg = "RTSP/1.0"
        if res is not None:
            msg += " {0}\r\nCSeq: {1}\r\n".format(res, seq)
        else:
            msg += "\r\nCSeq: {}\r\n".format(seq)
        if others is not None:
            for k, v in others:
                msg += "{}: {}\r\n".format(k, v)
        msg += "\r\n"
        return msg

    @staticmethod
    def _parse_transport_header(data: str) -> Tuple[bool, str, str]:
        """Parse Transport header value such as "Transport: RTP/AVP/UDP;unicast;client_port=1028;server_port=5000" """
        udp = True
        client_port = "0"
        server_port = "0"
        paramlist = data.split(";")
        for p in paramlist:
            if p.startswith("RTP"):
                rtp, avp, prot = p.split("/")
                if prot == "UDP":
                    udp = True
                elif prot == "TCP":
                    udp = False
                else:
                    raise ValueError
            elif p.startswith("unicast"):
                pass
            elif p.startswith("client_port"):
                _, client_port = p.split("=")
            elif p.startswith("server_port"):
                _, server_port = p.split("=")
            else:
                continue
        return udp, client_port, server_port

    def rtsp_m1(self) -> bool:
        headers = self.get_rtsp_headers()
        if headers["cmd"] != "OPTIONS":
            return False
        s_data = self._rtsp_response_header(
            seq=headers["CSeq"], res="200 OK", others=[("Public", "org.wfa.wfd1.0, SET_PARAMETER, GET_PARAMETER")]
        )
        self.logger.debug("<-{}".format(s_data))
        self.sock.write(s_data.encode("ASCII"))
        return True

    def rtsp_m2(self) -> bool:
        self.csnum = 100
        s_data = self._rtsp_response_header(
            seq=str(self.csnum), cmd="OPTIONS", url="*", others=[("Require", "org.wfa.wfd1.0")]
        )
        self.logger.debug("<-{}".format(s_data))
        self.sock.write(s_data.encode("ASCII"))
        headers = self.get_rtsp_headers()
        if headers["CSeq"] != "100" or headers["resp"] != "200 OK":
            return False
        return True

    def rtsp_m3(self) -> bool:
        headers = self.get_rtsp_headers()
        if headers["cmd"] != "GET_PARAMETER" or headers["url"] != "rtsp://localhost/wfd1.0":
            return False
        body = self.read_body(headers)
        msg = ""
        for req in body.decode("UTF-8").split("\r\n"):
            if req == "":
                continue
            elif req == "wfd_client_rtp_ports":
                msg += "wfd_client_rtp_ports: RTP/AVP/UDP;unicast {} 0 mode=play\r\n".format(self.config.rtp_port)
            elif req == "wfd_video_formats":
                msg += "wfd_video_formats: {}\r\n".format(self.wfd_video_formats)
            elif req in self.wfd_parameters:
                msg += "{}: {}\r\n".format(req, self.wfd_parameters[req])
            else:
                msg += "{}: none\r\n".format(req)

        m3resp = self._rtsp_response_header(
            seq=headers["CSeq"],
            res="200 OK",
            others=[("Content-Type", "text/parameters"), ("Content-Length", str(len(msg)))],
        )
        m3resp += msg
        self.logger.debug("<-{}".format(m3resp))
        self.sock.write(m3resp.encode("ASCII"))
        return True

    def rtsp_m4(self) -> bool:
        headers = self.get_rtsp_headers()
        if headers["cmd"] != "SET_PARAMETER" or headers["url"] != "rtsp://localhost/wfd1.0":
            return False
        self.read_body(headers)
        # FIXME: parse body here to retrieve video mode and set actual mode.
        s_data = self._rtsp_response_header(res="200 OK", seq=headers["CSeq"])
        self.logger.debug("<-{}".format(s_data))
        self.sock.write(s_data.encode("ASCII"))
        return True

    def rtsp_m5(self) -> bool:
        headers = self.get_rtsp_headers()
        self.read_body(headers)
        if headers["cmd"] != "SET_PARAMETER":
            self.logger.debug("M5: got other than SET_PARAMETER request.")
            s_data = self._rtsp_response_header(res="400 Bad Requests", seq=headers["CSeq"])
            self.logger.debug("<-{}".format(s_data))
            self.sock.write(s_data.encode("ASCII"))
            return False
        # FIXME: analyze body to have  'wfd_trigger_method: SETUP'
        s_data = self._rtsp_response_header(res="200 OK", seq=headers["CSeq"])
        self.logger.debug("<-{}".format(s_data))
        self.sock.write(s_data.encode("ASCII"))
        return True

    def rtsp_m6(self) -> Tuple[Optional[str], Optional[str]]:
        self.csnum += 1
        sessionid = None
        server_port = None
        m6req = self._rtsp_response_header(
            cmd="SETUP",
            url="rtsp://{0:s}/wfd1.0/streamid=0".format(self.config.peeraddress),
            seq=str(self.csnum),
            others=[("Transport", "RTP/AVP/UDP;unicast;client_port={0:d}".format(self.config.rtp_port))],
        )
        self.logger.debug("<-{}".format(m6req))
        self.sock.write(m6req.encode("ASCII"))

        headers = self.get_rtsp_headers()
        if headers["CSeq"] is not None and headers["CSeq"] != str(self.csnum):
            raise ValueError("Unmatch sequence number: {}".format(headers["CSeq"]))
        if "Transport" in headers:
            assert headers["Transport"] is not None
            udp, client_port, server_port = self._parse_transport_header(headers["Transport"])
            self.logger.debug("server port {}".format(server_port))
        if "Session" in headers:
            assert headers["Session"] is not None
            sessionid = headers["Session"].split(";")[0]
        return sessionid, server_port

    def rtsp_m7(self, sessionid: str) -> bool:
        self.csnum += 1
        m7req = self._rtsp_response_header(
            cmd="PLAY",
            url="rtsp://{0:s}/wfd1.0/streamid=0".format(self.config.peeraddress),
            seq=str(self.csnum),
            others=[("Session", sessionid)],
        )
        self.logger.debug("<-{}".format(m7req))
        self.sock.write(m7req.encode("ASCII"))
        headers = self.get_rtsp_headers()
        if headers["resp"] != "200 OK" or headers["CSeq"] != str(self.csnum):
            return False
        return True

    def negotiate(self) -> bool:
        self.logger.debug("---- Start negotiation ----")
        while True:
            if not self.rtsp_m1():
                break
            if not self.rtsp_m2():
                break
            if not self.rtsp_m3():
                break
            if not self.rtsp_m4():
                break
            if not self.rtsp_m5():
                break
            sessionid, server_port = self.rtsp_m6()
            if sessionid is None:
                break
            if not self.rtsp_m7(sessionid):
                break
            self.logger.info("---- Negotiation successful ----")
            return True
        self.logger.info("---- Negotiation failed ----")
        return False

    def is_keep_alive(self, headers):
        return headers["cmd"] == "GET_PARAMETER" and headers["url"] == "rtsp://localhost/wfd1.0"

    def is_parameter_change(self, headers):
        return headers["cmd"] == "SET_PARAMETER"

    def keep_alive(self, headers):
        self.read_body(headers)
        resp_msg = self._rtsp_response_header(seq=headers["CSeq"], res="200 OK")
        self.sock.write(resp_msg.encode("ASCII"))
        return

    def parameter_change(self, headers):
        body = self.read_body(headers)
        lines = body.decode("UTF-8").splitlines()
        if "wfd_trigger_method: TEARDOWN" in lines:
            self.logger.debug("Got TEARDOWN request.")
            self.response_teardown(headers)
            self.teardown = True
        return

    def response_teardown(self, headers):
        resp_msg = self._rtsp_response_header(seq=headers["CSeq"], res="200 OK")
        self.sock.write(resp_msg.encode("ASCII"))
        m5_msg = self._rtsp_response_header(seq=str(self.csnum), cmd="TEARDOWN", url="rtsp://localhost/wfd1.0")
        self.sock.write(m5_msg.encode("ASCII"))

    def is_response(self, headers):
        return headers["resp"] == "200 OK"

    def play(self) -> None:
        self.player.start()
        self.teardown = False
        self.watchdog = 0
        self.sock.settimeout(10)

        while True:
            try:
                headers = self.get_rtsp_headers()
            except socket.error as e:
                err = e.args[0]
                if err == errno.EAGAIN or err == errno.EWOULDBLOCK or err == errno.EALREADY or err == errno.EINPROGRESS:
                    sleep(0.1)
                    continue
                elif err == errno.ETIMEDOUT:
                    self.watchdog += 1
                    if self.watchdog > self.config.max_timeout:
                        break
                else:
                    self.logger.debug("Got unexpected socket error {}".format(e.args[0]))
                    break
            else:
                self.watchdog = 0
                if self.is_keep_alive(headers):
                    self.keep_alive(headers)
                elif self.is_parameter_change(headers):
                    self.parameter_change(headers)
                    if self.teardown:
                        self.player.stop()
                elif self.is_response(headers):
                    if self.teardown:
                        break
                else:
                    # ignore all other commands now...
                    continue

    def run(self):
        sd = ServiceDiscovery()
        sd.register()
        while True:
            self.sock = RTSPTransport(self.config.peeraddress, self.config.rtsp_port)
            if self.negotiate():
                self.play()
            self.sock.close()
            sleep(1)



#!/usr/bin/env python3

"""
picast - a simple wireless display receiver for Raspberry Pi

    Copyright (C) 2019 Hiroshi Miura
    Copyright (C) 2018 Hsun-Wei Cho

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import os
import subprocess
import tempfile
from logging import getLogger

from .settings import Settings


class Dhcpd:
    """DHCP server daemon running in background."""

    def __init__(self, interface: str, logger="picast"):
        """Constructor accept an interface to listen."""
        self.config = Settings()
        self.dhcpd = None
        self.interface = interface
        self.logger = getLogger(logger)
        self.conf_path = self._create_conf()

    def _create_conf(self):
        fd, conf_path = tempfile.mkstemp(suffix=".conf")
        conf = "start  {}\nend {}\ninterface {}\nnotify_file dumpleases\noption subnet {}\noption lease {}\n".format(
            Settings().peeraddress, Settings().peeraddress, self.interface, Settings().netmask, Settings().timeout
        )
        with open(conf_path, "w") as c:
            c.write(conf)
        return conf_path

    def start(self):
        self.logger.debug("Start dhcpd server.")
        self.dhcpd = subprocess.Popen(
            ["sudo", "udhcpd", "-f", self.conf_path], stdout=subprocess.PIPE, stderr=subprocess.PIPE
        )
        out, err = (None, None)
        try:
            out, err = self.dhcpd.communicate(timeout=1)
        except subprocess.TimeoutExpired:
            pass
        self.logger.debug("udhcpd return code: {}".format(self.dhcpd.returncode))
        self.logger.debug("udhcpd stdout: {}".format(out))
        self.logger.debug("udhcpd stderr: {}".format(err))
        if self.dhcpd.returncode is not None and self.dhcpd.returncode != 0:
            self.logger.fatal(
                "Failed to start udhcp, please check debug for reasons, e.g. port is in use by dnsmsq or something ..."
            )
            os._exit(1)

    def stop(self):
        if self.dhcpd is not None:
            self.dhcpd.terminate()
            self.conf_path.unlink()
            # FIXME: workaround for sudo process killing.
            os.system("sudo pkill udhcpd")



import asyncio
import socket
import threading
from time import sleep

import pytest

from picast.rtspsink import RtspSink, RTSPTransport
from picast.video import RasberryPiVideo


class MockPlayer():
    def __init__(self):
        pass

    def start(self):
        pass

    def stop(self):
        pass


class RtspSource(threading.Thread):

    def __init__(self, port):
        super(RtspSource, self).__init__()
        self.port = port
        self.status = True
        self.msg = None

    def run(self):
        self.sock =  socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.sock.bind(('127.0.0.1', self.port))
        self.sock.listen(1)
        conn, remote = self.sock.accept()
        conn.settimeout(1)

        # M1
        m1 = b"OPTIONS * RTSP/1.0\r\nCSeq: 0\r\nRequire: org.wfa.wfd1.0\r\n\r\n"
        conn.sendall(m1)
        sleep(0.01)
        expected = b"RTSP/1.0 200 OK\r\nCSeq: 0\r\nPublic: org.wfa.wfd1.0, SET_PARAMETER, GET_PARAMETER\r\n\r\n"
        m1_resp = conn.recv(len(expected))
        if m1_resp != expected:
            self.status = False
            self.msg = "M1 response failure: {}".format(m1_resp)
            conn.close()
            return
        # M2
        expected = b"OPTIONS * RTSP/1.0\r\nCSeq: 100\r\nRequire: org.wfa.wfd1.0\r\n\r\n"
        m2 = conn.recv(len(expected))
        if m2 != expected:
            resp_400 = b"RTSP/1.0 400 Bad Request\r\nCSeq: 100\r\n\r\n"
            conn.sendall(resp_400)
            self.status = False
            self.msg = "M2 request failure: {}".format(m2)
            sleep(0.01)
            conn.close()
            return
        m2_resp = b"RTSP/1.0 200 OK\r\nCSeq: 100\r\nPublic: org.wfa.wfd1.0, SETUP, TEARDOWN, PLAY, PAUSE, GET_PARAMETER, SET_PARAMETER\r\n\r\n"
        conn.sendall(m2_resp)

        # M3
        m3_body = "wfd_video_formats\r\nwfd_audio_codecs\r\nwfd_3d_video_formats\r\nwfd_content_protection\r\n" \
                  "wfd_display_edid\r\nwfd_coupled_sink\r\nwfd_client_rtp_ports\r\n"
        m3 = "GET_PARAMETER rtsp://localhost/wfd1.0 RTSP/1.0\r\nCSeq: 1\r\nContent-Type: text/parameters\r\n" \
             "Content-Length: {}\r\n\r\n{}".format(len(m3_body), m3_body).encode('ASCII')
        conn.sendall(m3)
        m3_resp = conn.recv(512).decode('UTF-8')
        if m3_resp != "RTSP/1.0 200 OK\r\nCSeq: 1\r\nContent-Type: text/parameters\r\nContent-Length: 304\r\n\r\n" \
                      "wfd_video_formats: 06 00 01 10 000101C3 00208006 00000000 00 0000 0000 00 none none\r\n" \
                      "wfd_audio_codecs: AAC 00000001 00, LPCM 00000002 00\r\n" \
                      "wfd_3d_video_formats: none\r\n" \
                      "wfd_content_protection: none\r\n" \
                      "wfd_display_edid: none\r\n" \
                      "wfd_coupled_sink: none\r\n" \
                      "wfd_client_rtp_ports: RTP/AVP/UDP;unicast 1028 0 mode=play\r\n":
            resp_400 = b"RTSP/1.0 400 Bad Request\r\nCSeq: 1\r\n\r\n"
            conn.sendall(resp_400)
            self.status = False
            self.msg = "M3 bad request: {}".format(m3_resp)
            sleep(1)
            conn.close()
            return
        # M4
        msg = "wfd_video_formats: 00 00 01 01 00000001 00000000 00000000 00 0000 0000 00 none none\r\nwfd_audio_codecs: LPCM 00000002 00\r\n" \
             "wfd_presentation_URL: rtsp://192.168.173.80/wfd1.0/streamid=0 none\r\n" \
             "wfd_client_rtp_ports: RTP/AVP/UDP;unicast 1028 0 mode=play\r\n"
        m4 = "SET_PARAMETER rtsp://localhost/wfd1.0 RTSP/1.0\r\nCSeq: 2\r\n" \
             "Content-Type: text/parameters\r\nContent-Length: {}\r\n\r\n".format(len(msg))
        m4 += msg
        conn.sendall(m4.encode('ASCII'))
        expected = b"RTSP/1.0 200 OK\r\nCSeq: 2\r\n\r\n"
        m4_resp = conn.recv(len(expected))
        if m4_resp != expected:
            self.status = False
            self.msg = "M4 bad response: {}".format(m4_resp)
            sleep(1)
            conn.close()
            return
        # M5
        body = "wfd_trigger_method: SETUP\r\n"
        m5 = "SET_PARAMETER rtsp://localhost/wfd1.0 RTSP/1.0\r\n" \
             "CSeq: 3\r\nContent-Type: text/paramters\r\nContent-Length: {}\r\n\r\n".format(len(body))
        m5 += body
        conn.sendall(m5.encode('ASCII'))
        expected = b"RTSP/1.0 200 OK\r\nCSeq: 3\r\n\r\n"
        m5_resp = conn.recv(len(expected))
        if m5_resp != expected:
            self.status = False
            self.msg = "M5 bad response: {}".format(m5_resp)
            sleep(1)
            conn.close()
            return
        sleep(0.01)
        # M6
        m6 = conn.recv(1000)
        if m6 != b"SETUP rtsp://192.168.173.80/wfd1.0/streamid=0 RTSP/1.0\r\nCSeq: 101\r\n" \
                     b"Transport: RTP/AVP/UDP;unicast;client_port=1028\r\n\r\n":
            resp_400 = b"RTSP/1.0 400 Bad Request\r\nCSeq: 101\r\n\r\n"
            conn.sendall(resp_400)
            self.status = False
            self.msg = "M6 request failure: {}".format(m6)
            sleep(1)
            conn.close()
            return
        m6_resp = "RTSP/1.0 200 OK\r\nCSeq: 101\r\nSession: 7C9C5678;timeout=30\r\n" \
                  "Transport: RTP/AVP/UDP;unicast;client_port=1028;server_port=5000\r\n\r\n"
        conn.sendall(m6_resp.encode('ASCII'))
        sleep(0.01)
        # M7
        m7 = conn.recv(200)
        if m7 != b"PLAY rtsp://192.168.173.80/wfd1.0/streamid=0 RTSP/1.0\r\nCSeq: 102\r\nSession: 7C9C5678\r\n\r\n":
            resp_400 = b"RTSP/1.0 400 Bad Request\r\nCSeq: 102\r\n\r\n"
            conn.sendall(resp_400)
            self.status = False
            self.msg = "M7 request failure: {}".format(m7)
            sleep(1)
            conn.close()
            return
        m7_resp = b"RTSP/1.0 200 OK\r\nCSeq: 102\r\n\r\n"
        conn.sendall(m7_resp)
        sleep(1)
        conn.close()

    def join(self, *args):
        threading.Thread.join(self, *args)
        return self.status, self.msg


@pytest.mark.connection
@pytest.mark.asyncio
async def test_rtsp_negotiation(monkeypatch, unused_port):

    def videomock(self):
        return "06 00 01 10 000101C3 00208006 00000000 00 0000 0000 00 none none"

    def nonemock(self, *args):
        return

    monkeypatch.setattr(RasberryPiVideo, "get_wfd_video_formats", videomock)
    monkeypatch.setattr(RasberryPiVideo, "_get_display_resolutions", nonemock)

    rtsp_source = RtspSource(unused_port)
    rtsp_source.start()
    sleep(0.5)
    player = MockPlayer()
    rtspsink = RtspSink(player)
    rtspsink.sock = RTSPTransport('127.0.0.1', unused_port)
    result = rtspsink.negotiate()
    status, msg = rtsp_source.join()
    if not status or not result:
        pytest.fail(msg)



import socket
import threading

import pytest

from picast.rtspsink import RtspSink, RTSPTransport
from picast.video import RasberryPiVideo


class MockServer(threading.Thread):

    def __init__(self, port: int, target: str = "open"):
        super(MockServer, self).__init__()
        self.port = port
        self.target = target
        self.status = True
        self.msg = None

    def run(self):
        self.sock =  socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.sock.bind(('127.0.0.1', self.port))
        self.sock.listen(1)
        conn, remote = self.sock.accept()

        if self.target == "open":
            pass
        elif self.target == "readline":
            m1 = b"OPTIONS * RTSP/1.0\r\nCSeq: 0\r\nRequire: org.wfa.wfd1.0\r\n\r\n"
            conn.sendall(m1)
        elif self.target == "m1":
            m1 = b"OPTIONS * RTSP/1.0\r\nCSeq: 0\r\nRequire: org.wfa.wfd1.0\r\n\r\n"
            conn.sendall(m1)
            m1_resp = conn.recv(1000)
            if m1_resp != b"RTSP/1.0 200 OK\r\nCSeq: 0\r\nPublic: org.wfa.wfd1.0, SET_PARAMETER, GET_PARAMETER\r\n\r\n":
                self.status = False
                self.msg = "M1 response failure: {}".format(m1_resp)
        elif self.target == "m2":
            m2 = conn.recv(1000)
            if m2 != b"OPTIONS * RTSP/1.0\r\nCSeq: 100\r\nRequire: org.wfa.wfd1.0\r\n\r\n":
                resp_400 = b"RTSP/1.0 400 Bad Request\r\nCSeq: 100\r\n\r\n"
                conn.sendall(resp_400)
                self.status = False
                self.msg = "M2 request failure: {}".format(m2)
            else:
                m2_resp = b"RTSP/1.0 200 OK\r\nCSeq: 100\r\nPublic: org.wfa.wfd1.0, SETUP, TEARDOWN, PLAY, PAUSE, GET_PARAMETER, SET_PARAMETER\r\n\r\n"
                conn.sendall(m2_resp)
        elif self.target == "m3":
            m3_body = "wfd_video_formats\r\nwfd_audio_codecs\r\nwfd_3d_video_formats\r\nwfd_content_protection\r\n" \
                      "wfd_display_edid\r\nwfd_coupled_sink\r\nwfd_client_rtp_ports\r\n"
            m3 = "GET_PARAMETER rtsp://localhost/wfd1.0 RTSP/1.0\r\nCSeq: 1\r\nContent-Type: text/parameters\r\n" \
                 "Content-Length: {}\r\n\r\n{}".format(len(m3_body), m3_body).encode('ASCII')
            conn.sendall(m3)
            m3_resp = conn.recv(1000).decode('UTF-8')
            if m3_resp != "RTSP/1.0 200 OK\r\nCSeq: 1\r\nContent-Type: text/parameters\r\nContent-Length: 304\r\n\r\n" \
                          "wfd_video_formats: 06 00 01 10 000101C3 00208006 00000000 00 0000 0000 00 none none\r\n" \
                          "wfd_audio_codecs: AAC 00000001 00, LPCM 00000002 00\r\n" \
                          "wfd_3d_video_formats: none\r\n" \
                          "wfd_content_protection: none\r\n" \
                          "wfd_display_edid: none\r\n" \
                          "wfd_coupled_sink: none\r\n" \
                          "wfd_client_rtp_ports: RTP/AVP/UDP;unicast 1028 0 mode=play\r\n":
                resp_400 = b"RTSP/1.0 400 Bad Request\r\nCSeq: 1\r\n\r\n"
                conn.sendall(resp_400)
                self.status = False
                self.msg = "M3 bad request: {}".format(m3_resp)
        elif self.target == "m4":
                msg = "wfd_video_formats: 00 00 01 01 00000001 00000000 00000000 00 0000 0000 00 none none\r\nwfd_audio_codecs: LPCM 00000002 00\r\n" \
                     "wfd_presentation_URL: rtsp://192.168.173.80/wfd1.0/streamid=0 none\r\n" \
                     "wfd_client_rtp_ports: RTP/AVP/UDP;unicast 1028 0 mode=play\r\n"
                m4 = "SET_PARAMETER rtsp://localhost/wfd1.0 RTSP/1.0\r\nCSeq: 2\r\n" \
                     "Content-Type: text/parameters\r\nContent-Length: {}\r\n\r\n".format(len(msg))
                m4 += msg
                conn.sendall(m4.encode("ASCII"))
                m4_resp = conn.recv(1000).decode('ASCII')
                if m4_resp != "RTSP/1.0 200 OK\r\nCSeq: 2\r\n\r\n":
                    self.status = False
                    self.msg = "M4 bad response: {}".format(m4_resp)
        elif self.target == "m5":
            body = "wfd_trigger_method: SETUP\r\n"
            m5 = "SET_PARAMETER rtsp://localhost/wfd1.0 RTSP/1.0\r\n" \
                 "CSeq: 3\r\nContent-Type: text/paramters\r\nContent-Length: {}\r\n\r\n".format(len(body))
            m5 += body
            conn.sendall(m5.encode('ASCII'))
            m5_resp = conn.recv(1000).decode('UTF-8')
            if m5_resp != "RTSP/1.0 200 OK\r\nCSeq: 3\r\n\r\n":
                self.status = False
                self.msg = "M5 bad response: {}".format(m5_resp)
        elif self.target == "m6":
            m6 = conn.recv(1000)
            if m6 != b"SETUP rtsp://192.168.173.80/wfd1.0/streamid=0 RTSP/1.0\r\nCSeq: 101\r\n" \
                     b"Transport: RTP/AVP/UDP;unicast;client_port=1028\r\n\r\n":
                resp_400 = b"RTSP/1.0 400 Bad Request\r\nCSeq: 101\r\n\r\n"
                conn.sendall(resp_400)
                self.status = False
                self.msg = "M6 request failure: {}".format(m6)
            else:
                m6_resp = b"RTSP/1.0 200 OK\r\nCSeq: 101\r\nSession: 7C9C5678;timeout=30\r\n" \
                          b"Transport: RTP/AVP/UDP;unicast;client_port=1028;server_port=5000\r\n\r\n"
                conn.sendall(m6_resp)
        elif self.target == "m7":
            m7 = conn.recv(1000)
            if m7 != b"PLAY rtsp://192.168.173.80/wfd1.0/streamid=0 RTSP/1.0\r\nCSeq: 102\r\nSession: 7C9C5678\r\n\r\n":
                resp_400 = b"RTSP/1.0 400 Bad Request\r\nCSeq: 102\r\n\r\n"
                conn.sendall(resp_400)
                self.status = False
                self.msg = "M7 request failure: {}".format(m7)
            else:
                m7_resp = b"RTSP/1.0 200 OK\r\nCSeq: 102\r\n\r\n"
                conn.sendall(m7_resp)
        else:
            pass
        conn.close()

    def join(self, *args):
        threading.Thread.join(self, *args)
        return self.status, self.msg


def test_open_connection(unused_port):
    server = MockServer(unused_port, target="open")
    server.start()
    transport = RTSPTransport('127.0.0.1', unused_port)
    assert transport is not None
    server.join()


def test_readline(unused_port):
    server = MockServer(unused_port, target="readline")
    server.start()
    transport = RTSPTransport('127.0.0.1', unused_port)
    line = transport.readline()
    assert line == b"OPTIONS * RTSP/1.0"
    server.join()
    transport.close()


def test_read_headers(monkeypatch, unused_port):
    def videomock(self):
        return "06 00 01 10 000101C3 00208006 00000000 00 0000 0000 00 none none"

    def nonemock(self, *args):
        return

    monkeypatch.setattr(RasberryPiVideo, "get_wfd_video_formats", videomock)
    monkeypatch.setattr(RasberryPiVideo, "_get_display_resolutions", nonemock)

    server = MockServer(unused_port, target="readline")
    server.start()
    player = None
    rtspsink = RtspSink(player)
    rtspsink.sock = RTSPTransport('127.0.0.1', unused_port)
    headers = rtspsink.read_headers()
    assert headers == ['OPTIONS * RTSP/1.0', 'CSeq: 0', 'Require: org.wfa.wfd1.0']
    server.join()
    rtspsink.sock.close()


def test_get_rtsp_headers(monkeypatch, unused_port):
    def videomock(self):
        return "06 00 01 10 000101C3 00208006 00000000 00 0000 0000 00 none none"
    def nonemock(self, *args):
        return
    monkeypatch.setattr(RasberryPiVideo, "get_wfd_video_formats", videomock)
    monkeypatch.setattr(RasberryPiVideo, "_get_display_resolutions", nonemock)

    server = MockServer(unused_port, target="readline")
    server.start()
    player = None
    rtspsink = RtspSink(player)
    rtspsink.sock = RTSPTransport('127.0.0.1', unused_port)
    headers = rtspsink.get_rtsp_headers()
    assert headers == {'CSeq': '0', 'Require': 'org.wfa.wfd1.0', 'cmd': 'OPTIONS', 'resp': None, 'url': '*'}
    server.join()


def test_rtsp_m1(monkeypatch, unused_port):
    def videomock(self):
        return "06 00 01 10 000101C3 00208006 00000000 00 0000 0000 00 none none"
    def nonemock(self, *args):
        return
    monkeypatch.setattr(RasberryPiVideo, "get_wfd_video_formats", videomock)
    monkeypatch.setattr(RasberryPiVideo, "_get_display_resolutions", nonemock)

    server = MockServer(unused_port, target="m1")
    server.start()
    player = None
    rtspsink = RtspSink(player)
    rtspsink.sock = RTSPTransport('127.0.0.1', unused_port)
    rtspsink.rtsp_m1()
    result, msg = server.join()
    if not result:
        pytest.fail(msg)


def test_rtsp_m2(monkeypatch, unused_port):
    def videomock(self):
        return "06 00 01 10 000101C3 00208006 00000000 00 0000 0000 00 none none"
    def nonemock(self, *args):
        return
    monkeypatch.setattr(RasberryPiVideo, "get_wfd_video_formats", videomock)
    monkeypatch.setattr(RasberryPiVideo, "_get_display_resolutions", nonemock)

    server = MockServer(unused_port, target="m2")
    server.start()
    player = None
    rtspsink = RtspSink(player)
    rtspsink.sock = RTSPTransport('127.0.0.1', unused_port)
    rtspsink.rtsp_m2()
    result, msg = server.join()
    if not result:
        pytest.fail(msg)


def test_rtsp_m3(monkeypatch, unused_port):
    def videomock(self):
        return "06 00 01 10 000101C3 00208006 00000000 00 0000 0000 00 none none"
    def nonemock(self, *args):
        return
    monkeypatch.setattr(RasberryPiVideo, "get_wfd_video_formats", videomock)
    monkeypatch.setattr(RasberryPiVideo, "_get_display_resolutions", nonemock)

    server = MockServer(unused_port, target="m3")
    server.start()
    player = None
    rtspsink = RtspSink(player)
    rtspsink.sock = RTSPTransport('127.0.0.1', unused_port)
    rtspsink.rtsp_m3()
    result, msg = server.join()
    if not result:
        pytest.fail(msg)


def test_rtsp_m4(monkeypatch, unused_port):
    def videomock(self):
        return "06 00 01 10 000101C3 00208006 00000000 00 0000 0000 00 none none"
    def nonemock(self, *args):
        return
    monkeypatch.setattr(RasberryPiVideo, "get_wfd_video_formats", videomock)
    monkeypatch.setattr(RasberryPiVideo, "_get_display_resolutions", nonemock)

    server = MockServer(unused_port, target="m4")
    server.start()
    player = None
    rtspsink = RtspSink(player)
    rtspsink.sock = RTSPTransport('127.0.0.1', unused_port)
    rtspsink.rtsp_m4()
    result, msg = server.join()
    if not result:
        pytest.fail(msg)


def test_rtsp_m5(monkeypatch, unused_port):
    def videomock(self):
        return "06 00 01 10 000101C3 00208006 00000000 00 0000 0000 00 none none"
    def nonemock(self, *args):
        return
    monkeypatch.setattr(RasberryPiVideo, "get_wfd_video_formats", videomock)
    monkeypatch.setattr(RasberryPiVideo, "_get_display_resolutions", nonemock)

    server = MockServer(unused_port, target="m5")
    server.start()
    player = None
    rtspsink = RtspSink(player)
    rtspsink.sock = RTSPTransport('127.0.0.1', unused_port)
    rtspsink.rtsp_m5()
    result, msg = server.join()
    if not result:
        pytest.fail(msg)


def test_rtsp_m6(monkeypatch, unused_port):
    def videomock(self):
        return "06 00 01 10 000101C3 00208006 00000000 00 0000 0000 00 none none"
    def nonemock(self, *args):
        return
    monkeypatch.setattr(RasberryPiVideo, "get_wfd_video_formats", videomock)
    monkeypatch.setattr(RasberryPiVideo, "_get_display_resolutions", nonemock)

    server = MockServer(unused_port, target="m6")
    server.start()
    player = None
    rtspsink = RtspSink(player)
    rtspsink.sock = RTSPTransport('127.0.0.1', unused_port)
    rtspsink.csnum = 100
    sessionid, serverport = rtspsink.rtsp_m6()
    result, msg = server.join()
    if not result:
        pytest.fail(msg)
    assert sessionid == '7C9C5678'
    assert serverport == '5000'


def test_rtsp_m7(monkeypatch, unused_port):
    def videomock(self):
        return "06 00 01 10 000101C3 00208006 00000000 00 0000 0000 00 none none"
    def nonemock(self, *args):
        return
    monkeypatch.setattr(RasberryPiVideo, "get_wfd_video_formats", videomock)
    monkeypatch.setattr(RasberryPiVideo, "_get_display_resolutions", nonemock)

    server = MockServer(unused_port, target="m7")
    server.start()
    player = None
    rtspsink = RtspSink(player)
    rtspsink.sock = RTSPTransport('127.0.0.1', unused_port)
    rtspsink.csnum = 101
    sessionid = '7C9C5678'
    rtspsink.rtsp_m7(sessionid)
    result, msg = server.join()
    if not result:
        pytest.fail(msg)
